##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF amazon webservice tool

$Id: AmazonTool.py,v 1.3 2003/11/24 13:42:57 ctheune Exp $"""

# Global Zope stuff
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

import zLOG
import time
import sys

import amazon
from amazon import AmazonError

permission_QueryAmazon = "Use Amazon WebService"

class AmazonTool(UniqueObject, SimpleItem):
    """An amazon webservice tool. Retrieves 
       site, testing their availability and notifying their owners."""

    id = 'portal_amazon'
    meta_type = 'CMF Amazon Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    manage_options = ( ({'label' : 'Overview', 'action' : 'manage_overview'},
                        {'label' : 'Configure', 'action' : 'manage_configure'}
                        ) + SimpleItem.manage_options )

    developerToken = ""
    associatesTag = "webservices-20"
    #
    # ZMI methods
    #
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile('www/explainAmazonTool', globals(),
                                        __name__='manage_overview')

    security.declareProtected(ManagePortal, 'manage_configure')
    manage_configure = PageTemplateFile('www/configureAmazonTool', globals(),
                                        __name__='manage_configure')

    security.declareProtected(ManagePortal, 'edit_configuration')
    def edit_configuration(self, developer_token, associate):
        """Update the configuration options."""
        self.setDeveloperToken(developer_token)
        self.setAssociatesTag(associate)
        if hasattr(self.REQUEST, 'RESPONSE'):
            self.REQUEST.RESPONSE.redirect('manage_configure')

    #
    # Interface methods
    #
    security.declareProtected(ManagePortal, 'setDeveloperToken')
    def setDeveloperToken(self, token):
        self.developerToken = token 

    security.declareProtected(ManagePortal, 'setAssociatesTag')
    def setAssociatesTag(self, tag):
        self.associatesTag = tag
    
    security.declareProtected(permission_QueryAmazon, 'isOnAmazon')
    def isOnAmazon(self, asin, locale=None):
        """Returns if the given ASIN is available on the given local
           catalog."""
        if not asin:
            return 0

        try:
            book = CachedAmazonFetcher.getBook(asin, locale, needPrice=0, license=self.developerToken, associate=self.associatesTag)
            return 1
        except:
            return 0
 
    security.declareProtected(permission_QueryAmazon, 'getBook')
    def getBook(self, asin, price=0, locale=None):
        """Fetch the book. (Cached)"""
        resultData = {}
        print self.associatesTag
        if self.isOnAmazon(asin, locale):
            try:    
                book = CachedAmazonFetcher.getBook(asin, locale, 
                    needPrice=price, license=self.developerToken, 
                    associate=self.associatesTag)
                resultData.update(book)
            except AmazonError, s:
                print s
                pass
        print resultData
        return resultData

InitializeClass(AmazonTool)

class CachedAmazonFetcher:
    """
        Fetches Amazon Book informations with respect to the Amazon License Agreement:

        - 24h caching of general data
        - 1h caching of pricings
    """

    cache = {}

    def getBook(self, asin, locale=None, needPrice=0, license=None, associate="webservices-20"):
        """
            Returns the book from the cache.
        """
        now = time.time()
        key = (asin, locale)
        if not self.cache.has_key(key):
            self.updateCache(asin, locale, license, associate)
        data, lastupdate = self.cache[key]
        lastupdatediff = (now - lastupdate) / (60 * 60)

        if lastupdatediff > (needPrice and 1 or 24):
            self.updateCache(asin, locale, license)
        if not self.cache.has_key(key):
            raise AmazonError, "No item found by this ASIN"

        return self.cache[key][0]

    def updateCache(self, asin, locale, license, associate="webservices-20"):
        """
            Fetches a book into the cache.
        """
        cache_key = (asin, locale)
        try:
            book = amazon.searchByASIN(asin, locale=locale, license_key=license, associate=associate)[0]
            book_ = {}
            for key in dir(book):
                book_[key] = getattr(book, key)
            if book_.has_key("Authors"):
                authors = book.Authors.Author
                if type(authors) == type([]):
                    book_['Authors'] = authors
                else:
                    book_['Authors'] = [authors, ]
            self.cache[cache_key] = (book_, time.time())
        except AmazonError, message:
            # When amazon returns None, we have to remove it from the cache
            self.cache[cache_key] = (None, time.time())
            zLOG.LOG("CMF Amazon",0,"Problem: %s, %s" % (sys.exc_type, sys.exc_value))
        except:
            zLOG.LOG("CMF Amazon",0,"Problem: %s, %s" % (sys.exc_type, sys.exc_value))


CachedAmazonFetcher = CachedAmazonFetcher()
